from __future__ import absolute_import

__copyright__ = """ Copyright (c) 2011-2013 Torsten Schmits

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
"""

import threading
import pickle
import re
import functools
import operator

import gobject

import dbus, dbus.service, dbus.mainloop
from dbus import Interface
from dbus.service import Object, method, BusName
from dbus.mainloop import glib

from tek import logger, debug
from tek.tools import find

from series.client.errors import SeriesClientException
import series.config

service_name = 'org.tek.eyepatch'
service_path = '/Service'
interface_name = 'org.tek.eyepatch.daemon'

class Service(Object):

    def __init__(self, queue, commit):
        self._queue = queue
        self._commit = commit
        self._init_dbus()

    def _init_dbus(self):
        session_bus = dbus.SessionBus()
        name = BusName(service_name, session_bus)
        super(Service, self).__init__(session_bus, service_path, name)

    @method(interface_name, in_signature='i')
    def watch(self, index):
        release = self._queue[index]
        text = 'DBus: Marking release {} as watched.'
        logger.info(text.format(release.release))
        release.watched = True
        self._commit(block=False)

    @method(interface_name, in_signature='is')
    def add_link_to_id(self, id, url):
        release = find(lambda r: r.id == id, self._queue)
        text = 'Adding link "{}" to release {}'.format(url, release.release)
        logger.info(text)
        release.add_link(url)
        self._commit(block=False)

    @method(interface_name)
    def pending_downloads(self):
        qualify = lambda r: not r.downloaded and not r.nuked
        releases = filter(qualify, self._queue)
        dls = [(r.id, r.release.search_string) for r in releases]
        return pickle.dumps(dls)

    @method(interface_name)
    def explain_downloadable(self):
        return '\n'.join(map(lambda r: r.explain_downloadable,
                             self._queue[-5:]))

    @method(interface_name, in_signature='sss')
    def _mark_downloaded_metadata(self, series, season, episode):
        monitor = self._find_by_metadata(series, season, episode)
        return self._mark_downloaded(monitor)

    @method(interface_name, in_signature='i')
    def _mark_downloaded_id(self, id):
        monitor = self._find_by_id(id)
        return self._mark_downloaded(monitor)

    def _mark_downloaded(self, monitor):
        if monitor:
            msg = 'Marking release "{}" as downloaded.'
            logger.info(msg.format(monitor.release))
            monitor.downloaded = True
            self._commit(block=False)
        return monitor is not None

    @method(interface_name, in_signature='s')
    def list_releases(self, regex):
        rex = re.compile(regex, re.I)
        matcher = lambda m: rex.search(str(m.release))
        matches =  filter(matcher, self._queue)
        return [[str(monitor.id), str(monitor.release)] for monitor in matches]

    def _find(self, matcher):
        return find(matcher, self._queue)

    def _find_by_id(self, id):
        return self._find(lambda r: r.id == id)

    def _find_by_metadata(self, series=None, season=None, episode=None):
        def matcher(monitor):
            release = monitor.release
            return (
                (not(series) or release.name == series) and
                (not(season) or str(release.season) == season) and
                (not(episode) or str(release.episode) == episode)
            )
        return self._find(matcher)

    @method(interface_name, in_signature='sss')
    def id_by_metadata(self, series, season, episode):
        results = self._find_by_metadata(series, season, episode)
        return results.id if results else -1

class DBusService(threading.Thread):

    def __init__(self, queue, commit):
        self._queue = queue
        self._commit = commit
        super(DBusService, self).__init__()
        self._mainloop = None

    def start(self):
        gobject.threads_init()
        glib.DBusGMainLoop(set_as_default=True)
        self.service = Service(self._queue, self._commit)
        self._mainloop = gobject.MainLoop()
        logger.info('Starting DBus service.')
        super(DBusService, self).start()

    def run(self):
        if self._mainloop is not None:
            self._mainloop.run()

    def stop(self):
        if self._mainloop is not None:
            self._mainloop.quit()

class Client(Interface):

    def __init__(self):
        bus = dbus.SessionBus()
        obj = bus.get_object(service_name, service_path)
        Interface.__init__(self, obj, interface_name)


class DBusClient(threading.Thread, Client):

    def _check_service(func):
        @functools.wraps(func)
        def wrapper(self, *a, **kw):
            return func(self, *a, **kw)
        return wrapper

    def __init__(self, async=True):
        threading.Thread.__init__(self)
        if not async:
            self._start_proxy()
        self._mainloop = None

    def run(self):
        gobject.threads_init()
        glib.DBusGMainLoop(set_as_default=True)
        self._start_proxy()
        self._mainloop = gobject.MainLoop()
        self._mainloop.run()

    def _start_proxy(self):
        try:
            logger.info('Starting DBus client.')
            Client.__init__(self)
        except dbus.DBusException as e:
            text = 'Service not running! ({})'.format(e.message)
            raise SeriesClientException(text)

    def stop(self):
        if self._mainloop is not None:
            self._mainloop.quit()

    @_check_service
    def add_link(self, url, series=None, season=None, episode=None):
        dls = pickle.loads(self.pending_downloads())
        if episode is not None:
            id = self.id_by_metadata(series, season, episode)
        else:
            release = find(lambda r: re.search(r[1], url, re.I), dls)
            id = release[0]
        if id != -1:
            self.add_link_to_id(id, url)
            logger.info('Added link to release.')
        else:
            logger.error('No matching release found!')

    def explain(self):
        logger.info(self.explain_downloadable())

    def mark_downloaded(self, series, season='', episode=''):
        is_id = series.isdigit()
        if is_id:
            success = self._mark_downloaded_id(int(series))
        else:
            success = self._mark_downloaded_metadata(series, season, episode)
        if success:
            logger.info('Success!')
        else:
            logger.error('Release not found!')

    def list(self, regex=''):
        matches = self.list_releases(regex)
        if matches:
            text = 'id #{}: {}'
            for id, description in matches:
                logger.info(text.format(id, description))
        else:
            logger.info('No matching release found.')

__all__ = ['DBusService', 'DBusClient']
