__copyright__ = """ Copyright (c) 2011-2013 Torsten Schmits

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
"""

import re

import requests

from tek import debug, logger
from tek.tools import find, resolve_redirect
from tek.config import lazy_configurable

from tek_utils.sharehoster import accepted_domains

from series.daemon.model.release import ReleaseFactory
from series.util import domain, is_torrent

class FeedEntry(object):

    def __init__(self, release, links=None, is_nuke=False, is_fix=False):
        self.release = release
        self.links = links
        self.is_nuke = is_nuke
        self.is_fix = is_fix
        self._fix = None

    def __repr__(self):
        from tek.tools import repr_params
        params = repr_params(self.release, self.links, self.is_nuke, self.is_fix)
        return '{}{}'.format(self.__class__.__name__, params)

    def __str__(self):
        return '{}: {}'.format(self.__class__.__name__, str(self.release))

    def __unicode__(self):
        return '{}: {}'.format(self.__class__.__name__, unicode(self.release))

    def nuke(self, entry):
        self.is_nuke = True
        self._fix = entry

    def resolve_redirects(self):
        for i, link in enumerate(self.links):
            if not is_torrent(link):
                self.links[i] = resolve_redirect(link)

class LinkFinder(object):

    def __init__(self, provider, _domain):
        self._provider = provider
        self._link_rex = '{} <a href="(?P<link>[^"]+)"'
        self._link_re = re.compile(self._link_rex.format(provider), re.I)
        dumb = ('{{}} {provider}[^<]+?<a[^>]*?href="(?P<link>[^"]+)"')
        self._link_rex_dumb = dumb.format(provider=provider)
        web = '<strong>{{}}.+?{provider}.+?<a[^>]*?href="(?P<link>[^"]+)"'
        self._link_rex_web = web.format(provider=provider)
        self._link_rex_lista = 'a href="(https?[^"]+{}[^"]+)"'
        # TODO default to existing functions?
        self.find_link_web = getattr(self, 'find_link_web_{}'.format(_domain),
                                      lambda *a: None)
        self.find_link_by_url = getattr(self,
                                         'find_link_by_url_{}'.format(_domain),
                                         lambda *a: None)

    def find_link_by_url(self, entry, title):
        try:
            matches = self._link_re.findall(entry.summary_detail.value)
            links = map(resolve_redirect, matches)
            return find(lambda s: title in s, links)
        except requests.RequestException as e:
            logger.error(e)

    def find_link_by_url_ddlvalley(self, entry, title):
        content = entry.content[0].value
        m = re.search(self._link_rex_web.format(title), content,
                      re.DOTALL | re.I)
        if m:
            return m.group('link')

    def find_link_by_url_wrzko(self, entry, title):
        return self.find_link_by_url_ddlvalley(entry, title)

    def find_link_dumb(self, entry, title):
        m = re.search(self._link_rex_dumb.format(title),
                      entry.summary_detail.value)
        if m:
            return m.group('link')

    def find_link_web(self, link, title, ignore):
        web = requests.get(link).content
        m = re.search(self._link_rex_web.format(title), web, re.DOTALL)
        if m:
            return m.group('link')

    def find_link_web_theextopia(self, link, title, search_string):
        web = requests.get(link).content
        temp = 'a href="(?P<link>[^"]+)".*>{providers}'
        lista = re.search(temp.format('|'.join(accepted_domains())), web, re.I)
        if lista:
            web = requests.get(lista.group(1)).content
            m = re.search(self._link_rex_lista.format(title), web, re.DOTALL |
                          re.I)
            if not m:
                m = re.search(self._link_rex_lista.format(search_string), web,
                              re.DOTALL | re.I)
            if m:
                return m.group('link')

class TorrentFinder(object):
    
    def __init__(self, _domain):
        self._link_rex = ''

    def find_link_by_url(self, entry, title):
        return entry.get('magneturi', entry.get('link', None))

    def find_link_dumb(self, entry, title):
        pass

    def find_link_web(self, entry, title, searchstring):
        pass

@lazy_configurable(daemon=['use_sharehosters', 'use_torrent'])
class FeedEntryFactory(object):

    _release_rex_classic = ('^(?P<name>.+?)'
                            '(\.(?P<enum>S?\d{1,2}?[xE]\d{1,2}?|))'
                            '(\..*?)??'
                            '(\.(?P<fix>((real\.)?proper|repack)))?'
                            '(\.(?P<res>\d{3,4}p))?'
                            '([._](?P<rtype>[^.]+))?'
                            '([._](?P<codec>[^-]+)'
                            '[-_](?P<group>[^.]+))?$')

    _release_rex_ezrss = ('^(?P<name>.+?)'
                          ' (?P<enum>\d{1,2}x\d{1,2})'
                          '\s*(\['
                          '(?P<res>\d{3,4}p)?'
                          '(( - )?[^ -\]]+)?'
                          '(( - )?(?P<group>[^ -\]]+))?'
                          '\])?(%%%%%%(?P<fix>)(?P<rtype>)(?P<codec>))?'
                          )

    def __init__(self, rss_url):
        _domain = domain(rss_url)
        release_rex = getattr(self, '_release_rex_' + _domain,
                              self._release_rex_classic)
        self._release_re = re.compile(release_rex, re.I)
        self._release_fact = ReleaseFactory()
        self._link_finders = []
        if self._use_sharehosters:
            self._link_finders += [LinkFinder(p, _domain) for p in
                                   accepted_domains()]
        if self._use_torrent:
            self._link_finders.append(TorrentFinder(_domain))

    def _release_from_title(self, title):
        match = self._release_re.match(title.lower())
        if match:
            groups = map(match.group, ('name', 'rtype', 'codec', 'group',
                                       'enum', 'res'))
            release = self._release_fact(title, *groups)
            return FeedEntry(release, is_fix=bool(match.group('fix')))
        else:
            text = u'Couldn\'t match title "{}"'.format(title)
            logger.error(text.encode('utf-8'))

    def _entry_from_feed(self, entry):
        releases = [t.strip() for t in entry.title.split('&')]
        return filter(None, map(self._release_from_title, releases))

    def series(self, entries, want_hd=True):
        is_series = lambda e: e.release.is_series and (e.release.is_hd or not
                                                       want_hd)
        for entry in entries:
            releases = self._entry_from_feed(entry)
            rel_entry = find(is_series, releases)
            if rel_entry:
                title = rel_entry.release.title
                rel_entry.links = self._find_links(entry, title, rel_entry)
                yield rel_entry
            else:
                msg = 'Discarding feed entries "{}" with releases:'
                logger.warn(msg.format(entry.title))
                for release in releases:
                    logger.warn(release.release)

    def _find_links(self, *a):
        return filter(None, [self._find_link(finder, *a) for finder in
                             self._link_finders])

    def _find_link(self, finder, entry, title, rel_entry):
        link = finder.find_link_by_url(entry, title)
        if link is None:
            link = finder.find_link_dumb(entry, title)
        if link is None:
            link = finder.find_link_web(entry.link, title,
                                        rel_entry.release.search_string)
        return link
