__copyright__ = """ Copyright (c) 2011-2013 Torsten Schmits

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
"""

import re

from quick_orm.core import Database

from sqlalchemy import Column, String, Integer, Boolean

import requests

from tek import logger
from tek.tools import camelcaseify
from tek.errors import MooException
from tek.config import ConfigClient

from tek_utils.sharehoster import downloader

from series import episode_enumeration, make_series_name
from series.util import is_torrent

class Release(object):
    __metaclass__ = Database.DefaultMeta
    title = Column(String)
    name = Column(String)
    codec = Column(String)
    group = Column(String)
    season = Column(Integer)
    episode = Column(Integer)
    is_series = Column(Boolean)
    resolution = Column(String)

    @property
    def is_hd(self):
        return self.resolution in ['720p', '1080p']

    def __repr__(self):
        from tek.tools import repr_params
        params = repr_params(self.title, self.name, self.codec, self.group,
                             (self.season, self.episode), self.is_series,
                             self.resolution)
        return '{}{}'.format(self.__class__.__name__, params)

    def __str__(self):
        return '{} {}x{}'.format(camelcaseify(self.name, sep=' '), self.season,
                                 self.episode)

    def __eq__(self, other):
        return isinstance(other, Release) and self.title == other.title

    def is_same_episode(self, other):
        return (isinstance(other, Release) and self.name == other.name and
                self.season == other.season and self.episode == other.episode)

    @property
    def search_string(self):
        sep = '[._ ]'
        fix = '({}[^._ ]+)?'.format(sep)
        name = sep.join(self.name.split('_'))
        enum = '{}s?{:0>2}e?{:0>2}'.format(sep, self.season, self.episode)
        res = '{}{}'.format(sep, self.resolution) if self.resolution else ''
        return '{}{}{}{}'.format(name, enum, fix, res)

@Database.foreign_key(Release)
@Database.foreign_key('ReleaseMonitor', ref_name='_fix', backref_name='_nuke')
class ReleaseMonitor(object):
    __metaclass__ = Database.DefaultMeta
    index = Column(Integer)
    downloaded = Column(Boolean)
    download_path = Column(String)
    archived = Column(Boolean)
    _nuked = Column(Boolean)
    _links = Column(String)
    watched = Column(Boolean)
    unknown = Column(Boolean)
    down = Column(Boolean)
    archiver_error = Column(Boolean)
    failed_downloads = Column(Integer)
    downloading = Column(Boolean)
    subtitles_downloaded = Column(Boolean)

    def __init__(self, release, index, links):
        self.release = release
        self.index = index
        self._links = '|$|'.join(links)
        self.downloaded = False
        self.download_path = ''
        self.archived = False
        self._nuked = False
        self.unknown = False
        self.down = False
        self.archiver_error = False
        self.failed_downloads = 0
        self.downloading = False
        self.subtitles_downloaded = False
        self.subs_missing = False
        self._all_links_dead = False
        self._new_links = True

    def nuke(self, fix):
        self._nuked = True
        self._fix = fix

    @property
    def nuked(self):
        return self._nuked

    @property
    def links(self):
        return self._links.split('|$|')

    @property
    def link(self):
        min_size = ConfigClient('daemon')('min_size')
        too_small = lambda d: d.file_size < min_size
        rex = re.compile('part\d+\.rar$')
        is_multipart = lambda d: d.file_path and rex.search(d.file_path)
        for link in self.links:
            if link:
                try:
                    if not is_torrent(link):
                        dl = downloader(link)
                        if too_small(dl) or is_multipart(dl):
                            raise MooException()
                except (MooException, requests.RequestException):
                    pass
                else:
                    return link

    @property
    def explain_downloadable(self):
        text = '{:s} is {}downloadable{}.'
        if self.downloadable:
            nope = ''
            suffix = ''
        else:
            nope = 'not '
            suffix = ' because it {}'
            reasons = [
                'is already downloaded' if self.downloaded else None,
                'is nuked' if self.nuked else None,
                'is down' if self.down else None,
                'is a download in progress' if self.downloading else None,
                ('has no valid links ({})'.format(','.join(self.links)) if
                    not bool(self.link) else None),
            ]
            reasons = filter(None, reasons)
            if len(reasons) > 1:
                reasons[-2:] = [' and '.join(reasons[-2:])]
            reason = ', '.join(reasons)
            suffix = suffix.format(reason)
        return text.format(self.release, nope, suffix)

    @property
    def downloadable(self):
        return (not (self.downloaded or self.nuked or self.down or
                     self.downloading or self.all_links_dead))

    def add_link(self, url):
        if self._links:
            self._links += '|$|'
        self._links += url
        self._all_links_dead = False
        self._new_links = True

    @property
    def all_links_dead(self):
        if getattr(self, '_new_links', True):
            self._all_links_dead = not bool(self.link)
            self._new_links = False
        return getattr(self, '_all_links_dead', False)

    @property
    def is_torrent(self):
        return is_torrent(self.link)

    def add_links_from_feed_entry(self, feed_entry):
        my_links = self.links
        for link in feed_entry.links:
            if link not in my_links:
                self.add_link(link)
        added = set(self.links) - set(my_links)
        if added:
            text = 'Added new links to release {}: {}'
            logger.info(text.format(self.release, ', '.join(added)))

class ReleaseFactory(object):

    def __call__(self, title, name, release_type, codec, group, enum=None, res=None):
        is_series = bool(enum)
        enum = episode_enumeration(enum)
        name = make_series_name(name)
        codec = codec or ''
        group = group or ''
        return Release(title=title, name=name, codec=codec.lower(),
                       group=group.lower(), season=enum[0], episode=enum[1],
                       is_series=is_series, resolution=res)

    def monitor(self, entry, index):
        entry.resolve_redirects()
        return ReleaseMonitor(entry.release, index, entry.links)
